
class DZMDispatcher{
    static Instance = null;
    static getInstance(){
        if(DZMDispatcher.Instance === null){
            DZMDispatcher.Instance = new DZMDispatcher();
        }
        return DZMDispatcher.Instance;
    }

    constructor() {
        console.log('Hello from DZMDispatcher')
        this.listeners = {};
        this.intersectionObserver = false;
        this.observersMap = new Map();
        if ('IntersectionObserver' in window) {
            this.intersectionObserver = new IntersectionObserver((entries, observer)=>{
                entries.forEach(entry => {
                    if (this.observersMap.has(entry.target)) {
                        const callback = this.observersMap.get(entry.target);
                        callback(entry.isIntersecting, entry);
                    }
                });
                }
            )
        }else{
            console.error('IntersectionObserver is not supported by your browser ')
        }


        this.events = {
            progressBar:{
                intersecting:'progressBar.intersecting'
            }
        }
        document.dispatchEvent(new CustomEvent('DZMDispatcherReady',{detail:{dispatcher:this}}))

    }

    on(event, callback) {
        if (!this.listeners[event]) {
            this.listeners[event] = [];
        }
        this.listeners[event].push(callback);
    }

    off(event, callback) {
        if (!this.listeners[event]) {
            return;
        }
        this.listeners[event] = this.listeners[event].filter(listener => listener !== callback);
    }

    dispatch(event, data, dispatchReadyEvent = false) {
        console.warn('DZMdispatched: ' + event,data)
        if(dispatchReadyEvent){
            document.dispatchEvent(new CustomEvent('DZMDispatcherReady',{detail:{dispatcher:this}}))

        }

        if (!this.listeners[event]) {
            return;
        }
        this.listeners[event].forEach(callback => callback(data));
    }

    observe(element, callback){
        // Save callback function to map
        this.observersMap.set(element, callback);
        // observe element
        this.intersectionObserver.observe(element);
    };
    observeIntersection(element,event,mode='both', oneTime = false){
        this.observersMap.set(element,(intersecting, entry)=>{
            if((mode === 'on')){
                if(intersecting)
                    this.dispatch(event,{element,intersecting,entry})

            }else if(mode === 'off'){
                if(!intersecting)
                    this.dispatch(event,{element,intersecting,entry})

            }else{
                this.dispatch(event,{element,intersecting,entry})

            }
            if(oneTime){
                this.intersectionObserver.unobserve(element)
                this.observersMap.delete(element)
            }
        })
        this.intersectionObserver.observe(element)
    }


}

window.dzmDispatcher = DZMDispatcher.getInstance()
